<?
  /**
  * The LFXlib Query Management Library
  * Bogdan Stancescu <bogdan@lanifex.com>, November 2004
  * $Id: LFX_query_manager.php,v 1.4 2005/03/29 09:27:22 bogdan Exp $
  * @package LFX
  */

  _checkMeOut(__FILE__);

    /**
    * This function builds an SQL string from an array which describes it.
    *
    * This is extremely useful when you have a page which issues the same
    * base SQL but with variations depending on a number of factors.
    *
    * The SQL is described by an associative array of the following form:
    * array(
    *   'select' => indexed array of elements to select (M)
    *   'from'   => indexed array which make up the FROM list (M)
    *   'join'   => indexed array of JOIN sub-arrays, each formatted like this:
    *     array(
    *       'type'      => the join type, typically 'LEFT' or 'RIGHT',
    *       'table'     => what table to perform this join with,
    *       'condition' => what conditions should be met by the elements,
    *     ),
    *   'where'  => associative array which describes the WHERE conditions:
    *     array(
    *       'type'       => how to "glue" the conditions, "AND" or "OR",
    *       'conditions' => indexed array of conditions to be glued; each
    *                       element in this array can be either a string
    *                       which describes a condition or another indexed
    *                       array like this one ('type','conditions'). Each
    *                       of the 'conditions' array can in turn be a string
    *                       or an associative array like this one, and so on.
    *     ),
    *   'group'  => an indexed array which contains GROUP BY elements as arrays:
    *     array(
    *       'field' => the field to group by,
    *       'type'  => the GROUP BY type ('ASC' or 'DESC') (optional)
    *     ),
    *   'order'  => an indexed array which contains ORDER BY elements with
    *               the same format as the GROUP BY elements above,
    *   'limit'  => associative array which describes the SQL LIMIT:
    *     array(
    *       'count'  => the maximum number of elements to return,
    *       'offset' => the offset (optional)
    *     )
    * )
    *
    * <b>Notes:</b>
    * - Elements marked with (M) are mandatory, all others are optional
    * - Elements explicitly marked as (optional) can miss from the respective
    *   structure, even if the structure is present (e.g. while the 'group'
    *   array is optional itself, even if you need it you can skip the 'type'
    *   entry from any of its elements).
    * - the "condition" element in the JOIN array can be either a plain string
    *   or a condition array with the same structure as the one used in WHERE.
    *
    * @param array $query associative array which describes the query
    * @return string the actual query
    */
    // TO DO: Parse JOIN conditions through LFX_buildSQLConditions
    //        (requires minor redesign of LFX_buildSQLConditions and
    //        LFX_buildSQL)

  function LFX_buildSQL($query)
  {
    $sql="SELECT\n";
    for($i=0;$i<count($query['select']);$i++) {
      $sql.='  '.$query['select'][$i].",\n";
    }
    $sql=substr($sql,0,-2)."\nFROM\n";
    for($i=0;$i<count($query['from']);$i++) {
      $sql.='  '.$query['from'][$i].",\n";
    }
    $i=0;
    $sql=substr($sql,0,-2)."\n";
    for ($i=0;$i<count($query['join']);$i++) {
      if (is_array($query['join'][$i]['condition'])) {
        //$myCondition=LFX_buildSQLConditions(array('conditions'=>$query['join'][$i]['condition']),1);
        $myCondition=LFX_buildSQLConditions($query['join'][$i]['condition'],1);
      } else {
        $myCondition=$query['join'][$i]['condition'];
      }
      $sql.=strtoupper($query['join'][$i]['type']).' JOIN '.
        $query['join'][$i]['table'].' ON '.$myCondition."\n";
    }
    if ($query['where']) {
      $sql.=LFX_buildSQLConditions($query['where']);
    }
    if ($query['group']) {
      $sql.="GROUP BY\n";
      for($i=0;$i<count($query['group']);$i++) {
        $sql.='  '.$query['group'][$i]['field'].' ';
        if ($query['group'][$i]['type']) {
          $sql.=strtoupper($query['group'][$i]['type']).' ';
        }
        $sql.=",\n";
      }
      $sql=substr($sql,0,-2)."\n";
    }
    if ($query['order']) {
      $sql.="ORDER BY\n";
      for($i=0;$i<count($query['order']);$i++) {
        $sql.='  '.$query['order'][$i]['field'].' ';
        if ($query['order'][$i]['type']) {
          $sql.=strtoupper($query['order'][$i]['type']).' ';
        }
        $sql.=",\n";
      }
      $sql=substr($sql,0,-2)."\n";
    }
    if ($query['limit']) {
      $sql.='LIMIT ';
      if ($query['limit']['offset']) {
        $sql.=$query['limit']['offset'].', '.$query['limit']['count'];
      } else {
        $sql.=$query['limit']['count'];
      }
    }
    return($sql);
  }

    /**
    * You shouldn't need to use this directly, it's an auxiliary function
    * for LFX_buildSQL which parses recursive conditions.
    *
    * @param array $where the conditions structure
    * @param integer $indent the indent (recursiveness) level, used to
    *   render the SQL nicely, with proper indentation
    * @return string the conditions parsed as SQL
    */
  function LFX_buildSQLConditions($where,$indent=0)
  {
    // Must first clear out empty arrays, as to avoid ending up with queries
    // such as "WHERE (id=5) AND () AND () AND (status!='resolved')".
    $where=LFX_cleanupSQLConditions($where);
    $sql='';
    $ind1=str_repeat('  ',$indent);
    $ind2=str_repeat('  ',$indent+1);
    for($i=0;$i<count($where['conditions']);$i++) {
      if (!$sql) {
        if (!$indent) {
          $sql="WHERE\n";
        }
      } else {
        $sql.=strtoupper($where['type'])." ";
      }
      $sql.="(\n".$ind2;
      if (is_string($where['conditions'][$i])) {
        $sql.=$where['conditions'][$i];
      } else {
        $sql.=LFX_buildSQLConditions($where['conditions'][$i],$indent+1);
      }
      $sql.="\n".$ind1.") ";
    }
    if ($sql && !$indent) {
      $sql.="\n";
    }
    //pecho("SQL is <b>$sql</b> after finishing array ".vardump($where)."<br />\n");
    return($sql);
  }

    /**
    * This function cleans up SQL conditions recursively.
    * SQL conditions of the form ((cond1) AND (() OR ()) AND (cond2)) are
    * returned as ((cond1) AND (cond2)).
    * @param array $where the SQL structure to clean up
    * @return array the cleaned up SQL conditions structure
    */
  function LFX_cleanupSQLConditions($where)
  {
    if (!is_array($where)) {
      return($where);
    }
    $newWhere=array('type'=>$where['type']);
    $success=false;
    for($i=0;$i<count($where['conditions']);$i++) {
      if($localWhere=LFX_cleanupSQLConditions($where['conditions'][$i])) {
        $newWhere['conditions'][]=$localWhere;
        $success=true;
      }
    }
    if ($success) {
      return($newWhere);
    } else {
      return(array());
    }
  }

?>
