<?

  /**
  * A class to provide locking functionality (as in managing lock files)
  * Bogdan Stancescu <bogdan@lanifex.com>, February 2003
  * Based on a script by Andrei Tinca <andrei@lanifex.com>
  * $Id: LFX_lock.php,v 1.6 2005/08/31 10:00:28 dieter Exp $
  * @package LFX
  */
    /**
    * Notes:
    * 1. All methods return NULL only if something went wrong - can't read file,
    *    illegal application name etc (application names must not contain slashes)
    * 2. If you specify an explicit application name in any of the methods, the
    *    current object's application will default to it in subsequent calls.
    *    This means you can start with $foo=new LFX_Lock("bar"); and then
    *    simply go on like "if ($foo->check())", "$foo->lock()", "$foo->unlock()" etc.
    */
 
  class LFX_Lock
  {

    var $appName;

      /**
      * The constructor - if an $appName is given, it will set it for all subsequent
      * method calls, as described in note #2 above
      */
    function LFX_Lock($appName=NULL)
    {
      if ($appName!==NULL) {
        $this->appName=$appName;
      }
    }

      /**
      * Functionally the core of this class, checks if the lock is in place.
      * Please mind the values this method returns!
      * @param string $appName the application name to check the lock for
      * @return mixed NULL if problems occur, false if there's no lock, the string 'locked'
      *   if the application is locked or the string 'dead' if the lock is in place but the
      *   application isn't running anymore. Please note checking for dead applications is
      *   done by checking if an instance of PHP is running with the pid gathered from the
      *   lock file, but no checking is performed on whether that instance of PHP is running
      *   exactly the current script (which means the $appName parameter used throughout
      *   this class doesn't necessarily have to be related to the script name).
      */
    function check($appName=NULL)
    {
      if (($appName===NULL) && (($appName=$this->appName)===NULL)) {
        return(NULL);
      }
      if (strpos(" ".$appName,"/")) {
        return(NULL);
      }
      $this->appName=$appName;
      if (is_file($this->file($appName))) {
        $pid=$this->getLockPid();
        $app=exec("ps --pid $pid -o \"%c\" --no-headers");
        if ((!$app) || (!strpos(' '.$app,'php'))) {
          // Lock exists but process dead
          return('dead');
        }
        return('locked');
      } else {
        return(false);
      }
    }

      /**
      * Used internally - returns the pid stored in the current application's lock file
      * @return string the pid or NULL if can't read file
      */
    function getLockPid()
    {
      $fp=@fopen($this->file($this->appName),'r');
      if (!$fp) {
        return(NULL);
      }
      $pid=fgets($fp);
      fclose($fp);
      return($pid);
    }

      /**
      * Gets an application name and returns the path to its lock;
      * may be orverrided by descendants.
      */
    function file($appName)
    {
      return("/tmp/$appName");
    }

      /**
      * Locks the current application or the one specified in the parameter if any.
      */
    function lock($appName=NULL)
    {
      if (($appName===NULL) && (($appName=$this->appName)===NULL)) {
        return(NULL);
      }
      $this->appName=$appName;
      $fp=fopen($this->file($appName),'a');
      if (!$fp) {
        return(NULL);
      }
      if (!flock($fp,LOCK_EX+LOCK_NB)) {
        fclose($fp);
        return(NULL);
      }
      clearstatcache();
      $size=filesize($this->file($appName)); 
      if ($size>0 || $size===FALSE) {
        flock($fp,LOCK_UN);
        fclose($fp);
        return(NULL);
      }
      fputs($fp,getMyPid());

      return(true);
    }

      /**
      * Unlocks the current application.
      * If you don't specify the first parameter, unlocking will only occur if the pid in the
      * lockfile is the same as the current process's.
      */
    function unlock($force=false, $appName=NULL)
    {
      if (($appName===NULL) && (($appName=$this->appName)===NULL)) {
        return(NULL);
      }
      if (strpos(" ".$appName,"/")) {
        return(NULL);
      }
      $this->appName=$appName;
      if (!is_file($this->file($appName))) {
        return(NULL);
      }
      $pid=$this->getLockPid();
      if ((!$force) && ($pid!=getMyPid())) {
        return(NULL);
      }
      $fp=@fopen($this->file($appName),'r');
      if (!$fp) {
        return(NULL);
      }
      if (!flock($fp,LOCK_UN+LOCK_NB)) {
        fclose($fp);
        return(NULL);
      }
      return(@unlink($this->file($appName)));
    }
  }

?>
